@@ -2,7 +2,7 @@
    NDBM interface for dbm use. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1990, 1991, 1993, 2007, 2011 Free Software Foundation,
+   Copyright (C) 1990, 1991, 1993, 2007, 2011, 2016 Free Software Foundation,
    Inc.
 
    GDBM is free software; you can redistribute it and/or modify
@@ -63,10 +63,15 @@ ndbm_open_dir_file0 (const char *file_name, int pagfd, int mode)
   int fd = -1;
   struct stat st, pagst;
   unsigned char dirbuf[DEF_DIR_SIZE];
+  int flags = (mode & GDBM_OPENMASK) == GDBM_READER ?
+                O_RDONLY : O_RDWR;
+
+  if (mode & GDBM_CLOEXEC)
+    flags |= O_CLOEXEC;
       
   if (fstat (pagfd, &pagst))
     {
-      gdbm_errno = GDBM_FILE_OPEN_ERROR; /* FIXME: special code? */
+      gdbm_set_errno (NULL, GDBM_FILE_OPEN_ERROR, TRUE); /* FIXME: special code? */
       return -1;
     } 
       
@@ -79,19 +84,19 @@ ndbm_open_dir_file0 (const char *file_name, int pagfd, int mode)
 	    {
 	      if (unlink (file_name))
 		{
-		  if (mode == GDBM_READER)
+		  if ((mode & GDBM_OPENMASK) == GDBM_READER)
 		    /* Ok, try to cope with it. */
 		    return pagfd;
 		  else
 		    {
-		      gdbm_errno = GDBM_FILE_OPEN_ERROR; 
+		      gdbm_set_errno (NULL, GDBM_FILE_OPEN_ERROR, TRUE); 
 		      return -1;
 		    } 
 		}
 	    }
 	  else
 	    {
-	      gdbm_errno = GDBM_FILE_OPEN_ERROR;
+	      gdbm_set_errno (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
 	      return -1;
 	    }
 	}
@@ -99,21 +104,21 @@ ndbm_open_dir_file0 (const char *file_name, int pagfd, int mode)
 	/* ok */;
       else if (st.st_size != DEF_DIR_SIZE)
 	{
-	  gdbm_errno = GDBM_BAD_MAGIC_NUMBER;
+	  gdbm_set_errno (NULL, GDBM_BAD_MAGIC_NUMBER, FALSE);
 	  return -1;
 	}
       else
 	{
-	  fd = open (file_name, O_RDWR);
+	  fd = open (file_name, flags);
 	  if (fd == -1)
 	    {
-	      gdbm_errno = GDBM_FILE_OPEN_ERROR;
+	      gdbm_set_errno (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
 	      return fd;
 	    }
 	  
 	  if (read (fd, dirbuf, sizeof (dirbuf)) != sizeof (dirbuf))
 	    {
-	      gdbm_errno = GDBM_FILE_OPEN_ERROR;
+	      gdbm_set_errno (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
 	      close (fd);
 	      return -1;
 	    } 
@@ -130,13 +135,13 @@ ndbm_open_dir_file0 (const char *file_name, int pagfd, int mode)
 		return fd;
 	    }
 	  close (fd);
-	  gdbm_errno = GDBM_BAD_MAGIC_NUMBER;
+	  gdbm_set_errno (NULL, GDBM_BAD_MAGIC_NUMBER, FALSE);
 	  return -1;
 	}
     }
   
   /* File does not exist.  Create it. */
-  fd = open (file_name, O_RDWR | O_CREAT, pagst.st_mode & 0777);
+  fd = open (file_name, flags | O_CREAT, pagst.st_mode & 0777);
   if (fd >= 0)
     {
       putint (dirbuf, GDBM_DIR_MAGIC);
@@ -146,7 +151,7 @@ ndbm_open_dir_file0 (const char *file_name, int pagfd, int mode)
 
       if (write (fd, dirbuf, sizeof (dirbuf)) != sizeof (dirbuf))
 	{
-	  gdbm_errno = GDBM_FILE_WRITE_ERROR;
+	  gdbm_set_errno (NULL, GDBM_FILE_WRITE_ERROR, FALSE);
 	  close (fd);
 	  fd = -1;
 	} 
@@ -163,7 +168,7 @@ ndbm_open_dir_file (const char *base, int pagfd, int mode)
   
   if (!file_name)
     {
-      gdbm_errno = GDBM_MALLOC_ERROR;
+      gdbm_set_errno (NULL, GDBM_MALLOC_ERROR, FALSE);
       return -1;
     }
   fd = ndbm_open_dir_file0 (strcat (strcpy (file_name, base), DIRSUF),
@@ -180,14 +185,20 @@ ndbm_open_dir_file (const char *base, int pagfd, int mode)
    size of zero bytes, a file initialization procedure is performed,
    setting up the initial structure in the file.  Any error detected will
    cause a return value of -1.  No errors cause the return value of 0.
-   NOTE: file.dir will be ignored and will always have a size of zero.
-   FLAGS and MODE are the same as the open (2) call.  This call will
+
+   The file.dir file is used only to ensure that the corresponding file.pag
+   was created using a compatible version of GDBM.  If file.dir is a hard
+   link to file.pag, as was in GDBM 1.8.x, it is removed and recreated in
+   the right format (see above).  If the database is being opened read-only,
+   the file.dir remains untouched.
+
+   FLAGS and MODE are the same as for the open(2) call.  This call will
    look at the FLAGS and decide what call to make to gdbm_open.  For
    FLAGS == O_RDONLY, it will be a GDBM_READER, if FLAGS == O_RDWR|O_CREAT,
    it will be a GDBM_WRCREAT (creater and writer) and if the FLAGS == O_RDWR,
    it will be a GDBM_WRITER and if FLAGS contain O_TRUNC then it will be
-   a GDBM_NEWDB.  All other values of FLAGS in the flags are
-   ignored. */
+   a GDBM_NEWDB.  The O_CLOEXEC bit raises GDBM_CLOEXEC flag.
+   All other values of FLAGS are ignored. */
 
 DBM *
 dbm_open (char *file, int flags, int mode)
@@ -195,12 +206,13 @@ dbm_open (char *file, int flags, int mode)
   char *pag_file;	    /* Used to construct "file.pag". */
   DBM *dbm = NULL;
   int open_flags;
+  int f;
   
   /* Prepare the correct name for "file.pag". */
-  pag_file = (char *) malloc (strlen (file)+5);
+  pag_file = (char *) malloc (strlen (file) + 5);
   if (!pag_file)
     {
-      gdbm_errno = GDBM_MALLOC_ERROR;	/* For the hell of it. */
+      gdbm_set_errno (NULL, GDBM_MALLOC_ERROR, FALSE); /* For the hell of it. */
       return NULL;
     }
 
@@ -208,18 +220,18 @@ dbm_open (char *file, int flags, int mode)
   strcat (pag_file, ".pag");
 
   /* Call the actual routine, saving the pointer to the file information. */
-  flags &= O_RDONLY | O_RDWR | O_CREAT | O_TRUNC;
+  f = flags & (O_RDONLY | O_RDWR | O_CREAT | O_TRUNC);
 
-  if (flags == O_RDONLY)
+  if (f == O_RDONLY)
     {
       open_flags = GDBM_READER;
       mode = 0;
     }
-  else if (flags == (O_RDWR | O_CREAT))
+  else if (f == (O_RDWR | O_CREAT))
     {
       open_flags = GDBM_WRCREAT;
     }
-  else if ((flags & O_TRUNC) == O_TRUNC)
+  else if ((f & O_TRUNC) == O_TRUNC)
     {
       open_flags = GDBM_NEWDB;
     }
@@ -229,20 +241,25 @@ dbm_open (char *file, int flags, int mode)
       mode = 0;
     }
 
+  if (flags & O_CLOEXEC)
+    open_flags |= GDBM_CLOEXEC;
+
+  open_flags |= GDBM_NOLOCK;
+  
   dbm = calloc (1, sizeof (*dbm));
   if (!dbm)
     {
       free (pag_file);
-      gdbm_errno = GDBM_MALLOC_ERROR;	/* For the hell of it. */
+      gdbm_set_errno (NULL, GDBM_MALLOC_ERROR, FALSE); /* For the hell of it. */
       return NULL;
     }
       
-  dbm->file = gdbm_open (pag_file, 0, open_flags | GDBM_NOLOCK, mode, NULL);
+  dbm->file = gdbm_open (pag_file, 0, open_flags, mode, NULL);
 
   /* Did we successfully open the file? */
   if (dbm->file == NULL)
     {
-      gdbm_errno = GDBM_FILE_OPEN_ERROR;
+      gdbm_set_errno (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
       free (dbm);
       dbm = NULL;
     }
