@@ -1,8 +1,8 @@
 /* gdbmdelete.c - Remove the key and its associated data from the database. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1990, 1991, 1993, 2007, 2011, 2013 Free Software Foundation,
-   Inc.
+   Copyright (C) 1990, 1991, 1993, 2007, 2011, 2013,
+   2016 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,28 +33,25 @@ gdbm_delete (GDBM_FILE dbf, datum key)
   int last_loc;		/* Last location emptied by the delete.  */
   int home;		/* Home position of an item. */
   bucket_element elem;  /* The element to be deleted. */
-  char *find_data;	/* Return pointer from findkey. */
-  int   hash_val;	/* Returned by findkey. */
-  off_t free_adr;       /* Temporary stroage for address and size. */
+  off_t free_adr;       /* Temporary storage for address and size. */
   int   free_size;
 
+  GDBM_ASSERT_CONSISTENCY (dbf, -1);
+
   /* First check to make sure this guy is a writer. */
   if (dbf->read_write == GDBM_READER)
     {
-      gdbm_errno = GDBM_READER_CANT_DELETE;
+      GDBM_SET_ERRNO (dbf, GDBM_READER_CANT_DELETE, FALSE);
       return -1;
     }
-
+  
   /* Initialize the gdbm_errno variable. */
-  gdbm_errno = GDBM_NO_ERROR;
+  gdbm_set_errno (dbf, GDBM_NO_ERROR, FALSE);
 
   /* Find the item. */
-  elem_loc = _gdbm_findkey (dbf, key, &find_data, &hash_val);
+  elem_loc = _gdbm_findkey (dbf, key, NULL, NULL);
   if (elem_loc == -1)
-    {
-      gdbm_errno = GDBM_ITEM_NOT_FOUND;
-      return -1;
-    }
+    return -1;
 
   /* Save the element.  */
   elem = dbf->bucket->h_table[elem_loc];
@@ -101,6 +98,5 @@ gdbm_delete (GDBM_FILE dbf, datum key)
   dbf->cache_entry->ca_data.elem_loc = -1;
 
   /* Do the writes. */
-  _gdbm_end_update (dbf);
-  return 0;  
+  return _gdbm_end_update (dbf);
 }
