@@ -1,7 +1,7 @@
 /* gdbmexp.c - Export a GDBM database. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 2007, 2011, 2013 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2011, 2013, 2016 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +32,9 @@ gdbm_export_to_file (GDBM_FILE dbf, FILE *fp)
   const char *header2 = "\r\n!\r\n";
   int count = 0;
 
+  /* Return immediately if the database needs recovery */	
+  GDBM_ASSERT_CONSISTENCY (dbf, -1);
+  
   /* Write out the text header. */
   if (fwrite (header1, strlen (header1), 1, fp) != 1)
     goto write_fail;
@@ -46,7 +49,12 @@ gdbm_export_to_file (GDBM_FILE dbf, FILE *fp)
   while (key.dptr != NULL)
     {
       data = gdbm_fetch (dbf, key);
-      if (data.dptr != NULL)
+      if (data.dptr == NULL)
+	{
+	  if (gdbm_errno != GDBM_NO_ERROR)
+	    return -1;
+	}
+      else
  	{
 	  /* Add the data to the new file. */
 	  size = htonl (key.dsize);
@@ -61,6 +69,7 @@ gdbm_export_to_file (GDBM_FILE dbf, FILE *fp)
 	  if (fwrite (data.dptr, data.dsize, 1, fp) != 1)
 	    goto write_fail;
  	}
+      
       nextkey = gdbm_nextkey (dbf, key);
       free (key.dptr);
       free (data.dptr);
@@ -68,12 +77,14 @@ gdbm_export_to_file (GDBM_FILE dbf, FILE *fp)
       
       count++;
     }
+  if (gdbm_errno != GDBM_ITEM_NOT_FOUND)
+    return -1;
   
   return count;
   
  write_fail:
   
-  gdbm_errno = GDBM_FILE_WRITE_ERROR;
+  GDBM_SET_ERRNO (NULL, GDBM_FILE_WRITE_ERROR, FALSE);
   return -1;
 }
 
@@ -90,7 +101,7 @@ gdbm_export (GDBM_FILE dbf, const char *exportfile, int flags, int mode)
       nfd = open (exportfile, O_WRONLY | O_CREAT | O_EXCL, mode);
       if (nfd == -1)
 	{
-	  gdbm_errno = GDBM_FILE_OPEN_ERROR;
+	  GDBM_SET_ERRNO (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
 	  return -1;
 	}
       break;
@@ -98,15 +109,15 @@ gdbm_export (GDBM_FILE dbf, const char *exportfile, int flags, int mode)
       nfd = open (exportfile, O_WRONLY | O_CREAT | O_TRUNC, mode);
       if (nfd == -1)
 	{
-	  gdbm_errno = GDBM_FILE_OPEN_ERROR;
+	  GDBM_SET_ERRNO (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
 	  return -1;
 	}
       break;
     default:
 #ifdef GDBM_BAD_OPEN_FLAGS
-      gdbm_errno = GDBM_BAD_OPEN_FLAGS;
+      GDBM_SET_ERRNO (NULL, GDBM_BAD_OPEN_FLAGS, FALSE);
 #else
-      gdbm_errno = GDBM_FILE_OPEN_ERROR;
+      GDBM_SET_ERRNO (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
 #endif
       return -1;
   }
@@ -115,7 +126,7 @@ gdbm_export (GDBM_FILE dbf, const char *exportfile, int flags, int mode)
   if (!fp)
     {
       close (nfd);
-      gdbm_errno = GDBM_FILE_OPEN_ERROR;
+      GDBM_SET_ERRNO (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
       return -1;
     }
 	
