@@ -1,7 +1,7 @@
 /* gdbmimp.c - Import a GDBM database. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 2007, 2011, 2013 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2011, 2013, 2016 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,6 +34,9 @@ gdbm_import_from_file (GDBM_FILE dbf, FILE *fp, int flag)
   datum key, data;
   int count = 0;
 
+  /* Return immediately if the database needs recovery */	
+  GDBM_ASSERT_CONSISTENCY (dbf, -1);
+  
   seenbang = 0;
   seennewline = 0;
   kbuffer = NULL;
@@ -44,7 +47,7 @@ gdbm_import_from_file (GDBM_FILE dbf, FILE *fp, int flag)
     {
       if ((rret = fgetc (fp)) == -1)
 	{
-	  gdbm_errno = GDBM_FILE_READ_ERROR;
+	  GDBM_SET_ERRNO (NULL, GDBM_FILE_READ_ERROR, FALSE);
 	  return -1;
 	}
       
@@ -62,19 +65,19 @@ gdbm_import_from_file (GDBM_FILE dbf, FILE *fp, int flag)
     }
 
   /* Allocate buffers. */
-  kbufsize = 512;
+  kbufsize = GDBM_MIN_BLOCK_SIZE;
   kbuffer = malloc (kbufsize);
   if (kbuffer == NULL)
     {
-      gdbm_errno = GDBM_MALLOC_ERROR;
+      GDBM_SET_ERRNO (NULL, GDBM_MALLOC_ERROR, FALSE);
       return -1;
     }
-  dbufsize = 512;
+  dbufsize = GDBM_MIN_BLOCK_SIZE;
   dbuffer = malloc (dbufsize);
   if (dbuffer == NULL)
     {
       free (kbuffer);
-      gdbm_errno = GDBM_MALLOC_ERROR;
+      GDBM_SET_ERRNO (NULL, GDBM_MALLOC_ERROR, FALSE);
       return -1;
     }
 
@@ -92,7 +95,7 @@ gdbm_import_from_file (GDBM_FILE dbf, FILE *fp, int flag)
       
       if (size > kbufsize)
 	{
-	  kbufsize = (size + 512);
+	  kbufsize = (size + GDBM_MIN_BLOCK_SIZE);
 	  kbuffer = realloc (kbuffer, kbufsize);
 	  if (kbuffer == NULL)
 	    {
@@ -124,7 +127,7 @@ gdbm_import_from_file (GDBM_FILE dbf, FILE *fp, int flag)
 	}
       if (size > dbufsize)
 	{
-	  dbufsize = (size + 512);
+	  dbufsize = (size + GDBM_MIN_BLOCK_SIZE);
 	  dbuffer = realloc (dbuffer, dbufsize);
 	  if (dbuffer == NULL)
 	    {
@@ -160,7 +163,7 @@ gdbm_import_from_file (GDBM_FILE dbf, FILE *fp, int flag)
   if (ec == GDBM_NO_ERROR)
     return count;
 
-  gdbm_errno = ec;
+  GDBM_SET_ERRNO (NULL, ec, FALSE);
   return -1;
 }
 
@@ -173,7 +176,7 @@ gdbm_import (GDBM_FILE dbf, const char *importfile, int flag)
   fp = fopen (importfile, "r");
   if (!fp)
     {
-      gdbm_errno = GDBM_FILE_OPEN_ERROR;
+      GDBM_SET_ERRNO (NULL, GDBM_FILE_OPEN_ERROR, FALSE);
       return -1;
     }
   rc = gdbm_import_from_file (dbf, fp, flag);
