@@ -1,6 +1,6 @@
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1990, 1991, 1993, 2007, 2011, 2013 Free Software Foundation,
-   Inc.
+   Copyright (C) 1990, 1991, 1993, 2007, 2011, 2013,
+   2016 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@ opendb (char *dbname)
   int cache_size = 0;
   int block_size = 0;
   int flags = 0;
+  int filemode;
   GDBM_FILE db;
   
   switch (variable_get ("cachesize", VART_INT, (void**) &cache_size))
@@ -85,7 +86,10 @@ opendb (char *dbname)
 	}
     }
   
-  db = gdbm_open (dbname, block_size, open_mode | flags, 0644, NULL);
+  if (variable_get ("filemode", VART_INT, (void**) &filemode))
+    abort ();
+
+  db = gdbm_open (dbname, block_size, open_mode | flags, filemode, NULL);
 
   if (db == NULL)
     {
@@ -127,30 +131,59 @@ bucket_print_lines (hash_bucket *bucket)
   return 6 + gdbm_file->header->bucket_elems + 3 + bucket->av_count;
 }
 
+static void
+format_key_start (FILE *fp, bucket_element *elt)
+{
+  int size = SMALL < elt->key_size ? SMALL : elt->key_size;
+  int i;
+
+  for (i = 0; i < size; i++)
+    {
+      if (isprint (elt->key_start[i]))
+	fprintf (fp, "   %c", elt->key_start[i]);
+      else
+	fprintf (fp, " %03o", elt->key_start[i]);
+    }
+}
+
 /* Debug procedure to print the contents of the current hash bucket. */
 void
-print_bucket (FILE *fp, hash_bucket *bucket, const char *mesg)
+print_bucket (FILE *fp, hash_bucket *bucket, const char *mesg, ...)
 {
-  int             index;
+  int index;
+  va_list ap;
 
+  fprintf (fp, "******* ");
+  va_start(ap, mesg);
+  vfprintf (fp, mesg, ap);
+  va_end (ap);
+  fprintf (fp, " **********\n\n");
   fprintf (fp,
-	   _("******* %s **********\n\nbits = %d\ncount= %d\nHash Table:\n"),
-	   mesg, bucket->bucket_bits, bucket->count);
+	   _("bits = %d\ncount= %d\nHash Table:\n"),
+	   bucket->bucket_bits, bucket->count);
   fprintf (fp,
-	   _("     #    hash value     key size    data size     data adr  home\n"));
+	   _("    #    hash value     key size    data size     data adr home  key start\n"));
   for (index = 0; index < gdbm_file->header->bucket_elems; index++)
-    fprintf (fp, "  %4d  %12x  %11d  %11d  %11lu %5d\n", index,
-	     bucket->h_table[index].hash_value,
-	     bucket->h_table[index].key_size,
-	     bucket->h_table[index].data_size,
-	     (unsigned long) bucket->h_table[index].data_pointer,
-	     bucket->h_table[index].hash_value %
-	     gdbm_file->header->bucket_elems);
+    {
+      fprintf (fp, " %4d  %12x  %11d  %11d  %11lu %4d", index,
+	       bucket->h_table[index].hash_value,
+	       bucket->h_table[index].key_size,
+	       bucket->h_table[index].data_size,
+	       (unsigned long) bucket->h_table[index].data_pointer,
+	       bucket->h_table[index].hash_value %
+	       gdbm_file->header->bucket_elems);
+      if (bucket->h_table[index].key_size)
+	{
+	  fprintf (fp, " ");
+	  format_key_start (fp, &bucket->h_table[index]);
+	}
+      fprintf (fp, "\n");
+    }
 
   fprintf (fp, _("\nAvail count = %1d\n"), bucket->av_count);
-  fprintf (fp, _("Avail  adr     size\n"));
+  fprintf (fp, _("Address           size\n"));
   for (index = 0; index < bucket->av_count; index++)
-    fprintf (fp, "%9lu%9d\n",
+    fprintf (fp, "%11lu%9d\n",
 	     (unsigned long) bucket->bucket_avail[index].av_adr,
 	     bucket->bucket_avail[index].av_size);
 }
@@ -266,7 +299,7 @@ _gdbm_print_bucket_cache (FILE *fp, GDBM_FILE dbf)
   if (dbf->bucket_cache != NULL)
     {
       fprintf (fp,
-	_("Bucket Cache (size %d):\n  Index:  Address  Changed  Data_Hash \n"),
+	_("Bucket Cache (size %zu):\n  Index:  Address  Changed  Data_Hash \n"),
 	 dbf->cache_size);
       for (index = 0; index < dbf->cache_size; index++)
 	{
@@ -319,46 +352,24 @@ get_screen_lines ()
 #endif
   return -1;
 }
-
-int
-get_record_count ()
-{
-  datum key, data;
-  int count = 0;
-  data = gdbm_firstkey (gdbm_file);
-  while (data.dptr != NULL)
-    {
-      count++;
-      key = data;
-      data = gdbm_nextkey (gdbm_file, key);
-      free (key.dptr);
-    }
-  return count;
-}
-
-
-#define ARG_UNUSED __attribute__ ((__unused__))
-
-#define NARGS 5
-
-struct handler_param
-{
-  int argc;
-  struct gdbmarg **argv;
-  FILE *fp;
-  void *data;
-};
-	     
 
 /* Open database */
 void
 open_handler (struct handler_param *param)
 {
-  if (opendb (param->argv[0]->v.string) == 0)
-    {
+  char *name = tildexpand (PARAM_STRING (param, 0));
+  if (file_name)
+    { 
+      gdbm_close (gdbm_file);
+      gdbm_file = NULL;
       free (file_name);
-      file_name = estrdup (param->argv[0]->v.string);
+      file_name = NULL;
     }
+  
+  if (opendb (name) == 0)
+    file_name = name;
+  else
+    free (name);
 }
 
 /* Close database */
@@ -367,31 +378,69 @@ close_handler (struct handler_param *param)
 {
   if (!gdbm_file)
     terror (_("nothing to close"));
-  gdbm_close (gdbm_file);
-  gdbm_file = NULL;
+  else
+    {
+      gdbm_close (gdbm_file);
+      gdbm_file = NULL;
+      free (file_name);
+      file_name = NULL;
+    }
 }
 
 
+static char *
+count_to_str (gdbm_count_t count, char *buf, size_t bufsize)
+{
+  char *p = buf + bufsize;
+
+  *--p = 0;
+  if (count == 0)
+    *--p = '0';
+  else
+    while (count)
+      {
+	if (p == buf)
+	  return NULL;
+	*--p = '0' + count % 10;
+	count /= 10;
+      }
+  return p;
+}
+  
 /* count - count items in the database */
 void
 count_handler (struct handler_param *param)
 {
-  int count = get_record_count ();
-  fprintf (param->fp, ngettext ("There is %d item in the database.\n",
-				"There are %d items in the database.\n", count),
-	   count);
+  gdbm_count_t count;
+
+  if (gdbm_count (gdbm_file, &count))
+    terror ("gdbm_count: %s", gdbm_strerror (gdbm_errno));
+  else
+    {
+      char buf[128];
+      char *p = count_to_str (count, buf, sizeof buf);
+
+      if (!p)
+	terror (_("count buffer overflow"));
+      else
+	fprintf (param->fp, 
+		 ngettext ("There is %s item in the database.\n",
+			   "There are %s items in the database.\n",
+			   count),
+		 p);
+    }
 }
 
 /* delete KEY - delete a key*/
 void
 delete_handler (struct handler_param *param)
 {
-  if (gdbm_delete (gdbm_file, param->argv[0]->v.dat) != 0)
+  if (gdbm_delete (gdbm_file, PARAM_DATUM (param, 0)) != 0)
     {
       if (gdbm_errno == GDBM_ITEM_NOT_FOUND)
-	terror (0, _("Item not found"));
+	terror (_("Item not found"));
       else
-	terror (0, _("Can't delete: %s"),  gdbm_strerror (gdbm_errno));
+	terror (_("Can't delete: %s"), gdbm_strerror (gdbm_errno));
     }
 }
 
@@ -399,15 +448,17 @@ delete_handler (struct handler_param *param)
 void
 fetch_handler (struct handler_param *param)
 {
-  return_data = gdbm_fetch (gdbm_file, param->argv[0]->v.dat);
+  return_data = gdbm_fetch (gdbm_file, PARAM_DATUM (param, 0));
   if (return_data.dptr != NULL)
     {
       datum_format (param->fp, &return_data, dsdef[DS_CONTENT]);
       fputc ('\n', param->fp);
       free (return_data.dptr);
     }
+  else if (gdbm_errno == GDBM_ITEM_NOT_FOUND)
+    terror ("%s", _("No such item found."));
   else
-    fprintf (stderr, _("No such item found.\n"));
+    terror (_("Can't fetch data: %s"), gdbm_strerror (gdbm_errno));
 }
 
 /* store KEY DATA - store data */
@@ -415,9 +466,9 @@ void
 store_handler (struct handler_param *param)
 {
   if (gdbm_store (gdbm_file,
-		  param->argv[0]->v.dat, param->argv[1]->v.dat,
+		  PARAM_DATUM (param, 0), PARAM_DATUM (param, 1),
 		  GDBM_REPLACE) != 0)
-    fprintf (stderr, _("Item not inserted.\n"));
+    terror ("%s", _("Item not inserted."));
 }
 
 /* first - begin iteration */
@@ -439,8 +490,10 @@ firstkey_handler (struct handler_param *param)
 
       free (return_data.dptr);
     }
-  else
+  else if (gdbm_errno == GDBM_ITEM_NOT_FOUND)
     fprintf (param->fp, _("No such item found.\n"));
+  else
+    terror (_("Can't find key: %s"), gdbm_strerror (gdbm_errno));
 }
 
 /* next [KEY] - next key */
@@ -451,9 +504,9 @@ nextkey_handler (struct handler_param *param)
     {
       if (key_data.dptr != NULL)
 	free (key_data.dptr);
-      key_data.dptr = emalloc (param->argv[0]->v.dat.dsize);
-      key_data.dsize = param->argv[0]->v.dat.dsize;
-      memcpy (key_data.dptr, param->argv[0]->v.dat.dptr, key_data.dsize);
+      key_data.dptr = emalloc (PARAM_DATUM (param, 0).dsize);
+      key_data.dsize = PARAM_DATUM (param, 0).dsize;
+      memcpy (key_data.dptr, PARAM_DATUM (param, 0).dptr, key_data.dsize);
     }
   return_data = gdbm_nextkey (gdbm_file, key_data);
   if (return_data.dptr != NULL)
@@ -468,27 +521,123 @@ nextkey_handler (struct handler_param *param)
 
       free (return_data.dptr);
     }
-  else
+  else if (gdbm_errno == GDBM_ITEM_NOT_FOUND)
     {
-      fprintf (stderr, _("No such item found.\n"));
+      terror ("%s", _("No such item found."));
       free (key_data.dptr);
       key_data.dptr = NULL;
     }
+  else
+    terror (_("Can't find key: %s"), gdbm_strerror (gdbm_errno));
 }
 
 /* reorganize */
 void
-reorganize_handler (struct handler_param *param ARG_UNUSED)
+reorganize_handler (struct handler_param *param GDBM_ARG_UNUSED)
 {
   if (gdbm_reorganize (gdbm_file))
-    fprintf (stderr, _("Reorganization failed.\n"));
+    terror ("%s", _("Reorganization failed."));
   else
-    fprintf (stderr, _("Reorganization succeeded.\n"));
+    fprintf (param->fp, _("Reorganization succeeded."));
 }
 
+static void
+err_printer (void *data GDBM_ARG_UNUSED, char const *fmt, ...)
+{
+  va_list ap;
+
+  va_start (ap, fmt);
+  vfprintf (stderr, fmt, ap);
+  va_end (ap);
+  fprintf (stderr, "\n");
+}
+
+/* recover verbose backup max-failed-keys=N max-failed-buckets=N max-failures=N */
+void
+recover_handler (struct handler_param *param)
+{
+  gdbm_recovery rcvr;
+  int flags = 0;
+  int rc;
+  int i;
+  char *p;
+  
+  for (i = 1; i < param->argc; i++)
+    {
+      char *arg = PARAM_STRING (param, i);
+      if (strcmp (arg, "verbose") == 0)
+	{
+	  rcvr.errfun = err_printer;
+	  flags |= GDBM_RCVR_ERRFUN;
+	}
+      else if (strcmp (arg, "backup") == 0)
+	{
+	  rcvr.errfun = err_printer;
+	  flags |= GDBM_RCVR_BACKUP;
+	}
+      else if (strncmp (arg, "max-failures=", 13) == 0)
+	{
+	  rcvr.max_failures = strtoul (arg + 13, &p, 10);
+	  if (*p)
+	    {
+	      printf (_("not a number (stopped near %s)\n"), p);
+	      return;
+	    }
+	  flags |= GDBM_RCVR_MAX_FAILURES;
+	}
+      else if (strncmp (arg, "max-failed-keys=", 16) == 0)
+	{
+	  rcvr.max_failed_keys = strtoul (arg + 16, &p, 10);
+	  if (*p)
+	    {
+	      printf (_("not a number (stopped near %s)\n"), p);
+	      return;
+	    }
+	  flags |= GDBM_RCVR_MAX_FAILED_KEYS;
+	}
+      else if (strncmp (arg, "max-failed-buckets=", 19) == 0)
+	{
+	  rcvr.max_failures = strtoul (arg + 19, &p, 10);
+	  if (*p)
+	    {
+	      printf (_("not a number (stopped near %s)\n"), p);
+	      return;
+	    }
+	  flags |= GDBM_RCVR_MAX_FAILED_BUCKETS;
+	}
+      else
+	{
+	  terror (_("unrecognized argument: %s"), arg);
+	  return;
+	}
+    }
+
+  rc = gdbm_recover (gdbm_file, &rcvr, flags);
+
+  if (rc == 0)
+    {
+      fprintf (param->fp, _("Recovery succeeded.\n"));
+      if (rcvr.backup_name)
+	{
+	  fprintf (param->fp,
+		   _("Original database preserved in file %s"),
+		   rcvr.backup_name);
+	  free (rcvr.backup_name);
+	}
+      fputc ('\n', param->fp);
+    }
+  else
+    {
+      fprintf (stderr, _("Recovery failed: %s"), gdbm_strerror (gdbm_errno));
+      if (gdbm_syserr[gdbm_errno])
+	fprintf (stderr, ": %s", strerror (errno));
+      fputc ('\n', stderr);
+    }
+}  
+
 /* avail - print available list */
 int
-avail_begin (struct handler_param *param ARG_UNUSED, size_t *exp_count)
+avail_begin (struct handler_param *param GDBM_ARG_UNUSED, size_t *exp_count)
 {
   if (checkdb ())
     return 1;
@@ -505,25 +654,37 @@ avail_handler (struct handler_param *param)
 
 /* C - print current bucket */
 int
-print_current_bucket_begin (struct handler_param *param ARG_UNUSED,
+print_current_bucket_begin (struct handler_param *param GDBM_ARG_UNUSED,
 			    size_t *exp_count)
 {
   if (checkdb ())
     return 1;
-  
+  if (!gdbm_file->bucket)
+    return 0;
   if (exp_count)
-    *exp_count = bucket_print_lines (gdbm_file->bucket) + 3;
+    *exp_count = gdbm_file->bucket
+                   ? bucket_print_lines (gdbm_file->bucket) + 3
+                   : 1;
   return 0;
 }
 
 void
 print_current_bucket_handler (struct handler_param *param)
 {
-  print_bucket (param->fp, gdbm_file->bucket, _("Current bucket"));
-  fprintf (param->fp, _("\n current directory entry = %d.\n"),
-	   gdbm_file->bucket_dir);
-  fprintf (param->fp, _(" current bucket address  = %lu.\n"),
-	   (unsigned long) gdbm_file->cache_entry->ca_adr);
+  if (!gdbm_file->bucket)
+    fprintf (param->fp, _("no current bucket\n"));
+  else
+    {
+      if (param->argc)
+	print_bucket (param->fp, gdbm_file->bucket, _("Bucket #%s"),
+		      PARAM_STRING (param, 0));
+      else
+	print_bucket (param->fp, gdbm_file->bucket, "%s", _("Current bucket"));
+      fprintf (param->fp, _("\n current directory entry = %d.\n"),
+	       gdbm_file->bucket_dir);
+      fprintf (param->fp, _(" current bucket address  = %lu.\n"),
+	       (unsigned long) gdbm_file->cache_entry->ca_adr);
+    }
 }
 
 int
@@ -559,12 +720,12 @@ print_bucket_begin (struct handler_param *param, size_t *exp_count)
   if (checkdb ())
     return 1;
   
-  if (getnum (&temp, param->argv[0]->v.string, NULL))
+  if (getnum (&temp, PARAM_STRING (param, 0), NULL))
     return 1;
 
-  if (temp >= gdbm_file->header->dir_size / 4)
+  if (temp >= GDBM_DIR_COUNT (gdbm_file))
     {
-      fprintf (stderr, _("Not a bucket.\n"));
+      terror (_("Not a bucket."));
       return 1;
     }
   _gdbm_get_bucket (gdbm_file, temp);
@@ -572,36 +733,54 @@ print_bucket_begin (struct handler_param *param, size_t *exp_count)
     *exp_count = bucket_print_lines (gdbm_file->bucket) + 3;
   return 0;
 }
-
 
 /* dir - print hash directory */
 int
-print_dir_begin (struct handler_param *param ARG_UNUSED, size_t *exp_count)
+print_dir_begin (struct handler_param *param GDBM_ARG_UNUSED, size_t *exp_count)
 {
   if (checkdb ())
     return 1;
   if (exp_count)
-    *exp_count = gdbm_file->header->dir_size / 4 + 3;
+    *exp_count = GDBM_DIR_COUNT (gdbm_file) + 3;
   return 0;
 }
 
+static size_t
+bucket_count (void)
+{
+  int i;
+  off_t last = 0;
+  size_t count = 0;
+  
+  for (i = 0; i < GDBM_DIR_COUNT (gdbm_file); i++)
+    {
+      if (gdbm_file->dir[i] != last)
+	{
+	  ++count;
+	  last = gdbm_file->dir[i];
+	}
+    }
+  return count;
+}
+
 void
 print_dir_handler (struct handler_param *param)
 {
   int i;
-
-  fprintf (param->fp, _("Hash table directory.\n"));
-  fprintf (param->fp, _("  Size =  %d.  Bits = %d. \n\n"),
-	   gdbm_file->header->dir_size, gdbm_file->header->dir_bits);
   
-  for (i = 0; i < gdbm_file->header->dir_size / 4; i++)
+  fprintf (param->fp, _("Hash table directory.\n"));
+  fprintf (param->fp, _("  Size =  %d.  Bits = %d,  Buckets = %zu.\n\n"),
+	   gdbm_file->header->dir_size, gdbm_file->header->dir_bits,
+	   bucket_count ());
+  
+  for (i = 0; i < GDBM_DIR_COUNT (gdbm_file); i++)
     fprintf (param->fp, "  %10d:  %12lu\n",
 	     i, (unsigned long) gdbm_file->dir[i]);
 }
 
 /* header - print file handler */
 int
-print_header_begin (struct handler_param *param ARG_UNUSED, size_t *exp_count)
+print_header_begin (struct handler_param *param GDBM_ARG_UNUSED, size_t *exp_count)
 {
   if (checkdb ())
     return 1;
@@ -636,13 +815,25 @@ print_header_handler (struct handler_param *param)
 void
 hash_handler (struct handler_param *param)
 {
-  fprintf (param->fp, _("hash value = %x. \n"),
-	   _gdbm_hash (param->argv[0]->v.dat));
+  if (gdbm_file)
+    {
+      int hashval, bucket, off;
+      _gdbm_hash_key (gdbm_file, PARAM_DATUM (param, 0),
+		       &hashval, &bucket, &off);
+      fprintf (param->fp, _("hash value = %x, bucket #%u, slot %u"),
+		hashval,
+		hashval >> (GDBM_HASH_BITS - gdbm_file->header->dir_bits),
+		hashval % gdbm_file->header->bucket_elems);
+    }
+  else
+    fprintf (param->fp, _("hash value = %x"),
+	      _gdbm_hash (PARAM_DATUM (param, 0)));
+  fprintf (param->fp, ".\n");
 }
 
 /* cache - print the bucket cache */
 int
-print_cache_begin (struct handler_param *param ARG_UNUSED, size_t *exp_count)
+print_cache_begin (struct handler_param *param GDBM_ARG_UNUSED, size_t *exp_count)
 {
   if (checkdb ())
     return 1;
@@ -650,7 +841,7 @@ print_cache_begin (struct handler_param *param ARG_UNUSED, size_t *exp_count)
     *exp_count = gdbm_file->bucket_cache ? gdbm_file->cache_size + 1 : 1;
   return 0;
 }
-    
+
 void
 print_cache_handler (struct handler_param *param)
 {
@@ -666,12 +857,22 @@ print_version_handler (struct handler_param *param)
 
 /* list - List all entries */
 int
-list_begin (struct handler_param *param ARG_UNUSED, size_t *exp_count)
+list_begin (struct handler_param *param GDBM_ARG_UNUSED, size_t *exp_count)
 {
   if (checkdb ())
     return 1;
   if (exp_count)
-    *exp_count = get_record_count ();
+    {
+      gdbm_count_t count;
+
+      if (gdbm_count (gdbm_file, &count))
+	 *exp_count = 0;
+      else if (count > SIZE_T_MAX)
+	 *exp_count = SIZE_T_MAX;
+      else
+	 *exp_count = count;
+    }
+
   return 0;
 }
 
@@ -688,18 +889,18 @@ list_handler (struct handler_param *param)
 
       data = gdbm_fetch (gdbm_file, key);
       if (!data.dptr)
-	{
-	  terror (_("cannot fetch data; the key was:"));
-	  datum_format (stderr, &key, dsdef[DS_KEY]);
-	}
+	 {
+	   terror (_("cannot fetch data; the key was:"));
+	   datum_format (stderr, &key, dsdef[DS_KEY]);
+	 }
       else
-	{
-	  datum_format (param->fp, &key, dsdef[DS_KEY]);
-	  fputc (' ', param->fp);
-	  datum_format (param->fp, &data, dsdef[DS_CONTENT]);
-	  fputc ('\n', param->fp);
-	  free (data.dptr);
-	}
+	 {
+	   datum_format (param->fp, &key, dsdef[DS_KEY]);
+	   fputc (' ', param->fp);
+	   datum_format (param->fp, &data, dsdef[DS_CONTENT]);
+	   fputc ('\n', param->fp);
+	   free (data.dptr);
+	 }
       free (key.dptr);
       key = nextkey;
     }
@@ -707,11 +908,11 @@ list_handler (struct handler_param *param)
 
 /* quit - quit the program */
 void
-quit_handler (struct handler_param *param ARG_UNUSED)
+quit_handler (struct handler_param *param GDBM_ARG_UNUSED)
 {
   if (gdbm_file != NULL)
     gdbm_close (gdbm_file);
-
+  input_done ();
   exit (EXIT_OK);
 }
 
@@ -722,27 +923,29 @@ export_handler (struct handler_param *param)
   int format = GDBM_DUMP_FMT_ASCII;
   int flags = GDBM_WRCREAT;
   int i;
-  
+  int filemode;
+
   for (i = 1; i < param->argc; i++)
     {
-      if (strcmp (param->argv[i]->v.string, "truncate") == 0)
-	flags = GDBM_NEWDB;
-      else if (strcmp (param->argv[i]->v.string, "binary") == 0)
-	format = GDBM_DUMP_FMT_BINARY;
-      else if (strcmp (param->argv[i]->v.string, "ascii") == 0)
-	format = GDBM_DUMP_FMT_ASCII;
+      if (strcmp (PARAM_STRING (param, i), "truncate") == 0)
+	 flags = GDBM_NEWDB;
+      else if (strcmp (PARAM_STRING (param, i), "binary") == 0)
+	 format = GDBM_DUMP_FMT_BINARY;
+      else if (strcmp (PARAM_STRING (param, i), "ascii") == 0)
+	 format = GDBM_DUMP_FMT_ASCII;
       else
-	{
-	  terror (_("unrecognized argument: %s"),
-			param->argv[i]->v.string);
-	  return;
-	}
+	 {
+	   terror (_("unrecognized argument: %s"), PARAM_STRING (param, i));
+	   return;
+	 }
     }
 
-  if (gdbm_dump (gdbm_file, param->argv[0]->v.string, format, flags, 0600))
+  if (variable_get ("filemode", VART_INT, (void**) &filemode))
+    abort ();
+  if (gdbm_dump (gdbm_file, PARAM_STRING (param, 0), format, flags, filemode))
     {
       terror (_("error dumping database: %s"),
-		    gdbm_strerror (gdbm_errno));
+		     gdbm_strerror (gdbm_errno));
     }
 }
 
@@ -755,23 +958,23 @@ import_handler (struct handler_param *param)
   int meta_mask = 0;
   int i;
   int rc;
-  
+
   for (i = 1; i < param->argc; i++)
     {
-      if (strcmp (param->argv[i]->v.string, "replace") == 0)
-	flag = GDBM_REPLACE;
-      else if (strcmp (param->argv[i]->v.string, "nometa") == 0)
-	meta_mask = GDBM_META_MASK_MODE | GDBM_META_MASK_OWNER;
+      if (strcmp (PARAM_STRING (param, i), "replace") == 0)
+	 flag = GDBM_REPLACE;
+      else if (strcmp (PARAM_STRING (param, i), "nometa") == 0)
+	 meta_mask = GDBM_META_MASK_MODE | GDBM_META_MASK_OWNER;
       else
-	{
-	  terror (_("unrecognized argument: %s"),
-			param->argv[i]->v.string);
-	  return;
-	}
+	 {
+	   terror (_("unrecognized argument: %s"),
+		   PARAM_STRING (param, i));
+	   return;
+	 }
     }
 
-  rc = gdbm_load (&gdbm_file, param->argv[0]->v.string, flag,
-		  meta_mask, &err_line);
+  rc = gdbm_load (&gdbm_file, PARAM_STRING (param, 0), flag,
+		   meta_mask, &err_line);
   if (rc && gdbm_errno == GDBM_NO_DBNAME)
     {
       int t = open_mode;
@@ -781,29 +984,29 @@ import_handler (struct handler_param *param)
       open_mode = t;
 
       if (rc)
-	return;
+	 return;
 
-      rc = gdbm_load (&gdbm_file, param->argv[0]->v.string, flag,
-		      meta_mask, &err_line);
+      rc = gdbm_load (&gdbm_file, PARAM_STRING (param, 0), flag,
+		       meta_mask, &err_line);
     }
   if (rc)
     {
       switch (gdbm_errno)
-	{
-	case GDBM_ERR_FILE_OWNER:
-	case GDBM_ERR_FILE_MODE:
-	  terror (_("error restoring metadata: %s (%s)"),
-			gdbm_strerror (gdbm_errno), strerror (errno));
-	  break;
-	  
-	default:
-	  if (err_line)
-	    terror ("%s:%lu: %s", param->argv[0], err_line,
-			  gdbm_strerror (gdbm_errno));
-	  else
-	    terror (_("cannot load from %s: %s"), param->argv[0],
-			  gdbm_strerror (gdbm_errno));
-	}
+	 {
+	 case GDBM_ERR_FILE_OWNER:
+	 case GDBM_ERR_FILE_MODE:
+	   terror (_("error restoring metadata: %s (%s)"),
+			 gdbm_strerror (gdbm_errno), strerror (errno));
+	   break;
+
+	 default:
+	   if (err_line)
+	     terror ("%s:%lu: %s", PARAM_STRING (param, 0), err_line,
+		     gdbm_strerror (gdbm_errno));
+	   else
+	     terror (_("cannot load from %s: %s"), PARAM_STRING (param, 0),
+		     gdbm_strerror (gdbm_errno));
+	 }
       return;
     }
 
@@ -828,10 +1031,81 @@ status_handler (struct handler_param *param)
   dsprint (param->fp, DS_CONTENT, dsdef[DS_CONTENT]);
 }
 
+#if GDBM_DEBUG_ENABLE
+static int
+debug_flag_printer (void *data, int flag, char const *tok)
+{
+  FILE *fp = data;
+  fprintf (fp, " %s", tok);
+  return 0;
+}
+#endif
+
+void
+debug_handler (struct handler_param *param)
+{
+#if GDBM_DEBUG_ENABLE
+  if (param->vararg)
+    {
+      struct gdbmarg *arg;
+      int i;
+      
+      for (arg = param->vararg, i = 0; arg; arg = arg->next, i++)
+	{
+	  if (arg->type == GDBM_ARG_STRING)
+	    {
+	      int flag;
+	      int negate;
+	      char const *tok = arg->v.string;
+	      
+	      if (tok[0] == '-')
+		{
+		  ++tok;
+		  negate = 1;
+		}
+	      else if (tok[0] == '+')
+		{
+		  ++tok;
+		  negate = 0;
+		}
+	      else
+		negate = 0;
+	      
+              flag = gdbm_debug_token (tok);
+	      if (flag)
+		{
+		  if (negate)
+		    gdbm_debug_flags &= ~flag;
+		  else
+		    gdbm_debug_flags |= flag;
+		}
+	      else
+		terror (_("unknown debug flag: %s"), tok);
+	    }
+	  else
+	    terror (_("invalid type of argument %d"), i);
+	}
+    }
+  else
+    {
+      fprintf (param->fp, _("Debug flags:"));
+      if (gdbm_debug_flags)
+	{
+	  gdbm_debug_parse_state (debug_flag_printer, param->fp);
+	}
+      else
+	fprintf (param->fp, " %s", _("none"));
+      fputc ('\n', param->fp);
+    }
+#else
+  terror ("%s", _("compiled without debug support"));
+#endif
+}
+
 void
 source_handler (struct handler_param *param)
 {
-  char *fname = tildexpand (param->argv[0]->v.string);
+  char *fname = tildexpand (PARAM_STRING (param, 0));
   if (setsource (fname, 0) == 0)
     yyparse ();
   free (fname);
@@ -848,6 +1122,15 @@ struct argdef
   int ds;
 };
 
+#define NARGS 10
+
+enum command_repeat_type
+  {
+    REPEAT_NEVER,
+    REPEAT_ALWAYS,
+    REPEAT_NOARG
+  };
+
 struct command
 {
   char *name;           /* Command name */
@@ -857,113 +1140,216 @@ struct command
   void (*handler) (struct handler_param *param);
   void (*end) (void *data);
   struct argdef args[NARGS];
+  int variadic;
+  enum command_repeat_type repeat;
   char *doc;
 };
-
 
 struct command command_tab[] = {
 #define S(s) #s, sizeof (#s) - 1
   { S(count), T_CMD,
     checkdb, count_handler, NULL,
-    { { NULL } }, N_("count (number of entries)") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("count (number of entries)") },
   { S(delete), T_CMD,
     checkdb, delete_handler, NULL,
-    { { N_("KEY"), ARG_DATUM, DS_KEY }, { NULL } }, N_("delete a record") },
+    { { N_("KEY"), GDBM_ARG_DATUM, DS_KEY }, { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("delete a record") },
   { S(export), T_CMD,
     checkdb, export_handler, NULL,
-    { { N_("FILE"), ARG_STRING },
-      { "[truncate]", ARG_STRING },
-      { "[binary|ascii]", ARG_STRING },
+    { { N_("FILE"), GDBM_ARG_STRING },
+      { "[truncate]", GDBM_ARG_STRING },
+      { "[binary|ascii]", GDBM_ARG_STRING },
       { NULL } },
+    FALSE,
+    REPEAT_NEVER,
     N_("export") },
   { S(fetch), T_CMD,
     checkdb, fetch_handler, NULL,
-    { { N_("KEY"), ARG_DATUM, DS_KEY }, { NULL } },  N_("fetch record") },
+    { { N_("KEY"), GDBM_ARG_DATUM, DS_KEY }, { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("fetch record") },
   { S(import), T_CMD,
     NULL, import_handler, NULL,
-    { { N_("FILE"), ARG_STRING },
-      { "[replace]", ARG_STRING },
-      { "[nometa]" , ARG_STRING },
+    { { N_("FILE"), GDBM_ARG_STRING },
+      { "[replace]", GDBM_ARG_STRING },
+      { "[nometa]" , GDBM_ARG_STRING },
       { NULL } },
+    FALSE,
+    FALSE,
     N_("import") },
   { S(list), T_CMD,
     list_begin, list_handler, NULL,
-    { { NULL } }, N_("list") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("list") },
   { S(next), T_CMD,
     checkdb, nextkey_handler, NULL,
-    { { N_("[KEY]"), ARG_STRING },
+    { { N_("[KEY]"), GDBM_ARG_DATUM, DS_KEY },
       { NULL } },
+    FALSE,
+    REPEAT_NOARG,
     N_("nextkey") },
   { S(store), T_CMD,
     checkdb, store_handler, NULL,
-    { { N_("KEY"), ARG_DATUM, DS_KEY },
-      { N_("DATA"), ARG_DATUM, DS_CONTENT },
+    { { N_("KEY"), GDBM_ARG_DATUM, DS_KEY },
+      { N_("DATA"), GDBM_ARG_DATUM, DS_CONTENT },
       { NULL } },
+    FALSE,
+    REPEAT_NEVER,
     N_("store") },
   { S(first), T_CMD,
     checkdb, firstkey_handler, NULL,
-    { { NULL } }, N_("firstkey") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("firstkey") },
   { S(reorganize), T_CMD,
     checkdb, reorganize_handler, NULL,
-    { { NULL } }, N_("reorganize") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("reorganize") },
+  { S(recover), T_CMD,
+    checkdb, recover_handler, NULL,
+    { { "[verbose]", GDBM_ARG_STRING },
+      { "[backup]",  GDBM_ARG_STRING },
+      { "[max-failed-keys=N]", GDBM_ARG_STRING },
+      { "[max-failed-buckets=N]", GDBM_ARG_STRING },
+      { "[max-failures=N]", GDBM_ARG_STRING },
+      { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("recover the database") },
   { S(avail), T_CMD,
     avail_begin, avail_handler, NULL,
-    { { NULL } }, N_("print avail list") }, 
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("print avail list") }, 
   { S(bucket), T_CMD,
     print_bucket_begin, print_current_bucket_handler, NULL,
-    { { N_("NUMBER"), ARG_STRING },
-      { NULL } }, N_("print a bucket") },
+    { { N_("NUMBER"), GDBM_ARG_STRING },
+      { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("print a bucket") },
   { S(current), T_CMD,
     print_current_bucket_begin, print_current_bucket_handler, NULL,
     { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
     N_("print current bucket") },
   { S(dir), T_CMD,
     print_dir_begin, print_dir_handler, NULL,
-    { { NULL } }, N_("print hash directory") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("print hash directory") },
   { S(header), T_CMD,
     print_header_begin , print_header_handler, NULL,
-    { { NULL } }, N_("print database file header") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("print database file header") },
   { S(hash), T_CMD,
     NULL, hash_handler, NULL,
-    { { N_("KEY"), ARG_DATUM, DS_KEY },
-      { NULL } }, N_("hash value of key") },
+    { { N_("KEY"), GDBM_ARG_DATUM, DS_KEY },
+      { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("hash value of key") },
   { S(cache), T_CMD,
     print_cache_begin, print_cache_handler, NULL,
-    { { NULL } }, N_("print the bucket cache") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("print the bucket cache") },
   { S(status), T_CMD,
     NULL, status_handler, NULL,
-    { { NULL } }, N_("print current program status") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("print current program status") },
   { S(version), T_CMD,
     NULL, print_version_handler, NULL,
-    { { NULL } }, N_("print version of gdbm") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("print version of gdbm") },
   { S(help), T_CMD,
     help_begin, help_handler, NULL,
-    { { NULL } }, N_("print this help list") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("print this help list") },
   { S(quit), T_CMD,
     NULL, quit_handler, NULL,
-    { { NULL } }, N_("quit the program") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("quit the program") },
   { S(set), T_SET,
     NULL, NULL, NULL,
-    { { "[VAR=VALUE...]" }, { NULL } }, N_("set or list variables") },
+    { { "[VAR=VALUE...]" }, { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("set or list variables") },
   { S(unset), T_UNSET,
     NULL, NULL, NULL,
-    { { "VAR..." }, { NULL } }, N_("unset variables") },
+    { { "VAR..." }, { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("unset variables") },
   { S(define), T_DEF,
     NULL, NULL, NULL,
-    { { "key|content", ARG_STRING },
-      { "{ FIELD-LIST }", ARG_STRING },
-      { NULL } }, N_("define datum structure") },
+    { { "key|content", GDBM_ARG_STRING },
+      { "{ FIELD-LIST }", GDBM_ARG_STRING },
+      { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("define datum structure") },
   { S(source), T_CMD,
     NULL, source_handler, NULL,
-    { { "FILE", ARG_STRING },
-      { NULL } }, N_("source command script") },
+    { { "FILE", GDBM_ARG_STRING },
+      { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("source command script") },
   { S(close), T_CMD,
     NULL, close_handler, NULL,
-    { { NULL } }, N_("close the database") },
+    { { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("close the database") },
   { S(open), T_CMD,
     NULL, open_handler, NULL,
-    { { "FILE", ARG_STRING }, { NULL } },
+    { { "FILE", GDBM_ARG_STRING }, { NULL } },
+    FALSE,
+    REPEAT_NEVER,
     N_("open new database") },
+#ifdef WITH_READLINE
+  { S(history), T_CMD,
+    input_history_begin, input_history_handler, NULL,
+    { { N_("[FROM]"), GDBM_ARG_STRING },
+      { N_("[COUNT]"), GDBM_ARG_STRING },
+      { NULL } },
+    FALSE,
+    REPEAT_NEVER,
+    N_("show input history") },
+#endif
+  { S(debug), T_CMD,
+    NULL, debug_handler, NULL,
+    { { NULL } },
+    TRUE,
+    REPEAT_NEVER,
+    N_("query/set debug level") },
 #undef S
   { 0 }
 };
@@ -983,12 +1369,45 @@ sort_commands ()
 	 sizeof (command_tab[0]), cmdcmp);
 }
 
+/* Generator function for command completion.  STATE lets us know whether
+   to start from scratch; without any state (i.e. STATE == 0), then we
+   start at the top of the list. */
+char *
+command_generator (const char *text, int state)
+{
+  const char *name;
+  static int len;
+  static struct command *cmd;
+
+  /* If this is a new word to complete, initialize now.  This includes
+     saving the length of TEXT for efficiency, and initializing the index
+     variable to 0. */
+  if (!state)
+    {
+      cmd = command_tab;
+      len = strlen (text);
+    }
+
+  if (!cmd->name)
+    return NULL;
+
+  /* Return the next name which partially matches from the command list. */
+  while ((name = cmd->name))
+    {
+      cmd++;
+      if (strncmp (name, text, len) == 0)
+        return strdup (name);
+    }
+
+  /* If no names matched, then return NULL. */
+  return NULL;
+}
 
 /* ? - help handler */
 #define CMDCOLS 30
 
 int
-help_begin (struct handler_param *param ARG_UNUSED, size_t *exp_count)
+help_begin (struct handler_param *param GDBM_ARG_UNUSED, size_t *exp_count)
 {
   if (exp_count)
     *exp_count = sizeof (command_tab) / sizeof (command_tab[0]) + 1;
@@ -1096,7 +1515,7 @@ gdbmarg_string (char *string, struct locus *loc)
 {
   struct gdbmarg *arg = ecalloc (1, sizeof (*arg));
   arg->next = NULL;
-  arg->type = ARG_STRING;
+  arg->type = GDBM_ARG_STRING;
   arg->ref = 1;
   if (loc)
     arg->loc = *loc;
@@ -1109,7 +1528,7 @@ gdbmarg_datum (datum *dat, struct locus *loc)
 {
   struct gdbmarg *arg = ecalloc (1, sizeof (*arg));
   arg->next = NULL;
-  arg->type = ARG_DATUM;
+  arg->type = GDBM_ARG_DATUM;
   arg->ref = 1;
   if (loc)
     arg->loc = *loc;
@@ -1122,7 +1541,7 @@ gdbmarg_kvpair (struct kvpair *kvp, struct locus *loc)
 {
   struct gdbmarg *arg = ecalloc (1, sizeof (*arg));
   arg->next = NULL;
-  arg->type = ARG_KVPAIR;
+  arg->type = GDBM_ARG_KVPAIR;
   arg->ref = 1;
   if (loc)
     arg->loc = *loc;
@@ -1131,7 +1550,7 @@ gdbmarg_kvpair (struct kvpair *kvp, struct locus *loc)
 }
 
 struct slist *
-slist_new (char *s)
+slist_new_s (char *s)
 {
   struct slist *lp = emalloc (sizeof (*lp));
   lp->next = NULL;
@@ -1139,6 +1558,21 @@ slist_new (char *s)
   return lp;
 }
 
+struct slist *
+slist_new (char const *s)
+{
+  return slist_new_s (estrdup (s));
+}
+
+struct slist *
+slist_new_l (char const *s, size_t l)
+{
+  char *copy = emalloc (l + 1);
+  memcpy (copy, s, l);
+  copy[l] = 0;
+  return slist_new_s (copy);
+}
+
 void
 slist_free (struct slist *lp)
 {
@@ -1150,6 +1584,21 @@ slist_free (struct slist *lp)
       lp = next;
     }
 }
+
+void
+slist_insert (struct slist **where, struct slist *what)
+{
+  if (*where)
+    {
+      while (what->next)
+	what = what->next;
+      what->next = (*where)->next;
+      (*where)->next = what;
+    }
+  else
+    what->next = NULL;
+  *where = what;
+}
 
 struct kvpair *
 kvpair_string (struct locus *loc, char *val)
@@ -1173,7 +1622,6 @@ kvpair_list (struct locus *loc, struct slist *s)
   return p;
 }  
 
-
 static void
 kvlist_free (struct kvpair *kvp)
 {
@@ -1203,15 +1651,15 @@ gdbmarg_free (struct gdbmarg *arg)
     {
       switch (arg->type)
 	{
-	case ARG_STRING:
+	case GDBM_ARG_STRING:
 	  free (arg->v.string);
 	  break;
 
-	case ARG_KVPAIR:
+	case GDBM_ARG_KVPAIR:
 	  kvlist_free (arg->v.kvpair);
 	  break;
 
-	case ARG_DATUM:
+	case GDBM_ARG_DATUM:
 	  free (arg->v.dat.dptr);
 	  break;
 	}
@@ -1258,6 +1706,7 @@ gdbmarglist_free (struct gdbmarglist *lst)
       gdbmarg_free (arg);
       arg = next;
     }
+  lst->head = lst->tail = NULL;
 }
 
 struct handler_param param;
@@ -1310,7 +1759,7 @@ coerce_s2d (struct gdbmarg *arg, struct argdef *def)
 
 #define coerce_fail NULL
 
-coerce_type_t coerce_tab[ARG_MAX][ARG_MAX] = {
+coerce_type_t coerce_tab[GDBM_ARG_MAX][GDBM_ARG_MAX] = {
   /*             s            d            k */
   /* s */  { coerce_ref,  coerce_fail, coerce_fail },
   /* d */  { coerce_s2d,  coerce_ref,  coerce_k2d }, 
@@ -1331,6 +1780,34 @@ coerce (struct gdbmarg *arg, struct argdef *def)
   return coerce_tab[def->type][arg->type] (arg, def);
 }
 
+static struct command *last_cmd;
+static struct gdbmarglist last_args;
+
+void
+run_last_command (void)
+{
+  if (interactive)
+    {
+      if (last_cmd)
+	{
+	  switch (last_cmd->repeat)
+	    {
+	    case REPEAT_NEVER:
+	      break;
+	    case REPEAT_NOARG:
+	      gdbmarglist_free (&last_args);
+	      /* FALLTHROUGH */
+	    case REPEAT_ALWAYS:
+	      if (run_command (last_cmd, &last_args))
+		exit (EXIT_USAGE);
+	      break;
+	    default:
+	      abort ();
+	    }
+	}
+    }
+}
+
 int
 run_command (struct command *cmd, struct gdbmarglist *arglist)
 {
@@ -1399,7 +1876,7 @@ run_command (struct command *cmd, struct gdbmarglist *arglist)
 	}
     }
 
-  if (arg)
+  if (arg && !cmd->variadic)
     {
       terror (_("%s: too many arguments"), cmd->name);
       return 1;
@@ -1413,6 +1890,7 @@ run_command (struct command *cmd, struct gdbmarglist *arglist)
       param.argv = ecalloc (argmax, sizeof (param.argv[0]));
     }
   param.argv[i] = NULL;
+  param.vararg = arg;
   param.fp = NULL;
   param.data = NULL;
   pagfp = NULL;
@@ -1448,6 +1926,13 @@ run_command (struct command *cmd, struct gdbmarglist *arglist)
     }
 
   param_free_argv (&param, param.argc);
+
+  last_cmd = cmd;
+  if (arglist->head != last_args.head)
+    {
+      gdbmarglist_free (&last_args);
+      last_args = *arglist;
+    }
   
   return 0;
 }
@@ -1484,6 +1969,18 @@ source_rcfile ()
     }
 }
 
+#if GDBM_DEBUG_ENABLE
+void
+debug_printer (char const *fmt, ...)
+{
+  va_list ap;
+
+  va_start (ap, fmt);
+  vfprintf (stderr, fmt, ap);
+  va_end (ap);
+}
+#endif
+
 int
 main (int argc, char *argv[])
 {
@@ -1491,9 +1988,13 @@ main (int argc, char *argv[])
   int opt;
   int bv;
   int norc = 0;
+  int res;
   char *source = "-";
   
   set_progname (argv[0]);
+#if GDBM_DEBUG_ENABLE
+  gdbm_debug_printer = debug_printer;
+#endif
 
 #ifdef HAVE_SETLOCALE
   setlocale (LC_ALL, "");
@@ -1502,15 +2003,18 @@ main (int argc, char *argv[])
   textdomain (PACKAGE);
 
   sort_commands ();
-
+  
   /* Initialize variables. */
   intr = isatty (0);
+  interactive = intr; /* Used early by input_init */
   dsdef[DS_KEY] = dsegm_new_field (datadef_lookup ("string"), NULL, 1);
   dsdef[DS_CONTENT] = dsegm_new_field (datadef_lookup ("string"), NULL, 1);
 
   variable_set ("open", VART_STRING, "wrcreat");
   variable_set ("pager", VART_STRING, getenv ("PAGER"));
   
+  input_init ();
+  
   for (opt = parseopt_first (argc, argv, optab);
        opt != EOF;
        opt = parseopt_next ())
@@ -1557,7 +2061,7 @@ main (int argc, char *argv[])
 	break;
 	
       case 'g':
-	file_name = optarg;
+	file_name = estrdup (optarg);
 	break;
 
       case 'q':
@@ -1570,7 +2074,7 @@ main (int argc, char *argv[])
 		progname);
 	exit (EXIT_USAGE);
       }
-
+  
   argc -= optind;
   argv += optind;
 
@@ -1581,7 +2085,7 @@ main (int argc, char *argv[])
     }
       
   if (argc == 1)
-    file_name = argv[0];
+    file_name = estrdup (argv[0]);
 
   signal (SIGPIPE, SIG_IGN);
 
@@ -1597,5 +2101,7 @@ main (int argc, char *argv[])
 
   if (setsource (source, intr))
     exit (EXIT_FATAL);
-  return yyparse ();
+  res = yyparse ();
+  input_done ();
+  return res;
 }
