@@ -24,6 +24,15 @@
 #include <stdarg.h>
 #include <ctype.h>
 
+#ifndef GDBM_ARG_UNUSED
+# define GDBM_ARG_UNUSED __attribute__ ((__unused__))
+#endif
+
+#ifndef GDBM_PRINTFLIKE
+# define GDBM_PRINTFLIKE(fmt,narg) \
+  __attribute__ ((__format__ (__printf__, fmt, narg)))
+#endif
+
 /* Position in input file */
 struct point
 {
@@ -90,11 +99,13 @@ typedef struct locus gdbm_yyltype_t;
   while (0)
 
 void vlerror (struct locus *loc, const char *fmt, va_list ap);
-void lerror (struct locus *loc, const char *fmt, ...);
+void lerror (struct locus *loc, const char *fmt, ...)
+	   GDBM_PRINTFLIKE (2, 3);
 
-void terror (const char *fmt, ...);
+void terror (const char *fmt, ...)
+	   GDBM_PRINTFLIKE (1, 2);
 
-void print_prompt (void);
+char *make_prompt (void);
 
 int setsource (const char *filename, int intr);
 
@@ -104,6 +115,17 @@ extern int open_mode;
 
 #define GDBMTOOLRC ".gdbmtoolrc"
 #define GDBMTOOL_DEFFILE "junk.gdbm"
+
+ssize_t input_read (FILE *fp, char *buf, size_t size);
+void input_init (void);
+void input_done (void);
+
+struct handler_param;
+void input_history_handler (struct handler_param *param);
+int input_history_begin (struct handler_param *param, size_t *exp_count);
+
+void print_prompt_at_bol (void);
+char *command_generator (const char *text, int state);
 
 
 struct slist
@@ -112,8 +134,11 @@ struct slist
   char *str;
 };
 
-struct slist *slist_new (char *s);
+struct slist *slist_new (char const *s);
+struct slist *slist_new_s (char *s);
+struct slist *slist_new_l (char const *s, size_t l);
 void slist_free (struct slist *);
+void slist_insert (struct slist **where, struct slist *what);
 
 #define KV_STRING 0
 #define KV_LIST   1
@@ -161,6 +186,19 @@ struct gdbmarglist
   struct gdbmarg *head, *tail;
 };
 
+struct handler_param
+{
+  int argc;
+  struct gdbmarg **argv;
+  struct gdbmarg *vararg;
+  FILE *fp;
+  void *data;
+};
+
+#define PARAM_STRING(p,n) ((p)->argv[n]->v.string)
+#define PARAM_DATUM(p,n)  ((p)->argv[n]->v.dat)
+#define PARAM_KVPAIR(p,n) ((p)->argv[n]->v.kvpair)
+
 void gdbmarglist_init (struct gdbmarglist *, struct gdbmarg *);
 void gdbmarglist_add (struct gdbmarglist *, struct gdbmarg *);
 void gdbmarglist_free (struct gdbmarglist *lst);
@@ -176,6 +214,7 @@ struct command;
 int command_lookup (const char *str, struct locus *loc, struct command **pcmd);
 
 int run_command (struct command *cmd, struct gdbmarglist *arglist);
+void run_last_command (void);
 
 struct xdatum;
 void xd_expand (struct xdatum *xd, size_t size);
@@ -261,4 +300,7 @@ void dsprint (FILE *fp, int what, struct dsegm *ds);
 char *mkfilename (const char *dir, const char *file, const char *suf);
 char *tildexpand (char *s);
 int vgetyn (const char *prompt, va_list ap);
-int getyn (const char *prompt, ...);
+int getyn (const char *prompt, ...) GDBM_PRINTFLIKE (1, 2);
+
+int getnum (int *pnum, char *arg, char **endp);
+int get_screen_lines (void);
