@@ -20,12 +20,12 @@
 
 /* From bucket.c */
 void _gdbm_new_bucket	(GDBM_FILE, hash_bucket *, int);
-void _gdbm_get_bucket	(GDBM_FILE, int);
+int _gdbm_get_bucket	(GDBM_FILE, int);
 int _gdbm_read_bucket_at (GDBM_FILE dbf, off_t off, hash_bucket *bucket,
 			  size_t size);
 
-void _gdbm_split_bucket (GDBM_FILE, int);
-void _gdbm_write_bucket (GDBM_FILE, cache_elem *);
+int _gdbm_split_bucket (GDBM_FILE, int);
+int _gdbm_write_bucket (GDBM_FILE, cache_elem *);
 
 /* From falloc.c */
 off_t _gdbm_alloc       (GDBM_FILE, int);
@@ -38,9 +38,12 @@ int _gdbm_findkey       (GDBM_FILE, datum, char **, int *);
 
 /* From hash.c */
 int _gdbm_hash (datum);
+void _gdbm_hash_key (GDBM_FILE dbf, datum key, int *hash, int *bucket,
+		     int *offset);
+int _gdbm_bucket_dir (GDBM_FILE dbf, int hash);
 
 /* From update.c */
-void _gdbm_end_update   (GDBM_FILE);
+int _gdbm_end_update   (GDBM_FILE);
 void _gdbm_fatal	(GDBM_FILE, const char *);
 
 /* From gdbmopen.c */
@@ -73,5 +76,7 @@ int _gdbm_base64_decode (const unsigned char *input, size_t input_len,
 int _gdbm_load (FILE *fp, GDBM_FILE *pdbf, unsigned long *line);
 int _gdbm_dump (GDBM_FILE dbf, FILE *fp);
 
+/* From recover.c */
+int _gdbm_next_bucket_dir (GDBM_FILE dbf, int bucket_dir);
 
 
